const semver = require('semver');

class VersionCheck {
  equal(v1, v2) {
    return semver.eq(v1, v2);
  }

  notEqual(v1, v2) {
    return semver.neq(v1, v2);
  }

  greaterThan(v1, v2) {
    return semver.gt(v1, v2);
  }

  greaterThanEqual(v1, v2) {
    return semver.gte(v1, v2);
  }

  lesserThan(v1, v2) {
    return semver.lt(v1, v2);
  }

  lesserThanEqual(v1, v2) {
    return semver.lte(v1, v2);
  }

  sort(v1, v2) {
    if(v1 === v2) return 0;
    if(semver.valid(v1) && semver.valid(v2)) return semver.rcompare(v1, v2);

    const parts = v1.split('.');
    const v1Patch = parts[parts.length -1];

    const version1 = semver.coerce(v1);
    const version2 = semver.coerce(v2);
    const coercedCompare = semver.rcompare(version1, version2);

    if(coercedCompare === 0) {
      const isBiggerPatch = this.isLatest(v1Patch);
      return isBiggerPatch ? -1 : 1;
    }

    return coercedCompare;
  }

  isLatest(patch) {
    return ['*', 'x'].includes(patch.toLowerCase());
  }
}

module.exports = new VersionCheck();
